@@ -18,10 +18,14 @@ import uuid
 import warnings
 from collections import OrderedDict
 
-from flask import current_app, request, url_for
+from flask import current_app, request, url_for, Flask
 from werkzeug.utils import import_string
+from flask_caching.backends.base import BaseCache
+from flask_caching.backends.simplecache import SimpleCache
+from markupsafe import Markup
+from typing import Any, Callable, List, Optional, Tuple, Union
 
-__version__ = "1.9.0"
+__version__ = "1.10.0"
 
 logger = logging.getLogger(__name__)
 
@@ -41,16 +45,15 @@ delchars = "".join(c for c in map(chr, range(256)) if c not in valid_chars)
 null_control = (dict((k, None) for k in delchars),)
 
 
-def wants_args(f):
-    """Check if the function wants any arguments
-    """
+def wants_args(f: Callable) -> bool:
+    """Check if the function wants any arguments"""
 
     argspec = inspect.getfullargspec(f)
 
     return bool(argspec.args or argspec.varargs or argspec.varkw)
 
 
-def get_arg_names(f):
+def get_arg_names(f: Callable) -> List[str]:
     """Return arguments of function
 
     :param f:
@@ -64,7 +67,7 @@ def get_arg_names(f):
     ]
 
 
-def get_arg_default(f, position):
+def get_arg_default(f: Callable, position: int):
     sig = inspect.signature(f)
     arg = list(sig.parameters.values())[position]
     arg_def = arg.default
@@ -132,7 +135,9 @@ def function_namespace(f, args=None):
     return ns, ins
 
 
-def make_template_fragment_key(fragment_name, vary_on=[]):
+def make_template_fragment_key(
+    fragment_name: str, vary_on: List[str] = []
+) -> str:
     """Make a cache key for a specific fragment name."""
     if vary_on:
         fragment_name = "%s_" % fragment_name
@@ -142,7 +147,12 @@ def make_template_fragment_key(fragment_name, vary_on=[]):
 class Cache(object):
     """This class is used to control the cache objects."""
 
-    def __init__(self, app=None, with_jinja2_ext=True, config=None):
+    def __init__(
+        self,
+        app: Optional[Flask] = None,
+        with_jinja2_ext: bool = True,
+        config=None,
+    ) -> None:
         if not (config is None or isinstance(config, dict)):
             raise ValueError("`config` must be an instance of dict or None")
 
@@ -154,7 +164,7 @@ class Cache(object):
         if app is not None:
             self.init_app(app, config)
 
-    def init_app(self, app, config=None):
+    def init_app(self, app: Flask, config=None) -> None:
         """This is used to initialize cache with your app object"""
         if not (config is None or isinstance(config, dict)):
             raise ValueError("`config` must be an instance of dict or None")
@@ -209,23 +219,28 @@ class Cache(object):
 
         self._set_cache(app, config)
 
-    def _set_cache(self, app, config):
+    def _set_cache(self, app: Flask, config) -> None:
         import_me = config["CACHE_TYPE"]
         if "." not in import_me:
-            from . import backends
-
-            try:
-                cache_obj = getattr(backends, import_me)
-            except AttributeError:
-                raise ImportError(
-                    "%s is not a valid Flask-Caching backend" % (import_me)
-                )
+            plain_name_used = True
+            import_me = type(self).__module__ + ".backends." + import_me
         else:
-            cache_obj = import_string(import_me)
+            plain_name_used = False
 
+        cache_factory = import_string(import_me)
         cache_args = config["CACHE_ARGS"][:]
         cache_options = {"default_timeout": config["CACHE_DEFAULT_TIMEOUT"]}
 
+        if isinstance(cache_factory, type) and issubclass(cache_factory, BaseCache):
+            cache_factory = cache_factory.factory
+        elif plain_name_used:
+            warnings.warn(
+                "Using the initialization functions in flask_caching.backend "
+                "is deprecated.  Use the a full path to backend classes "
+                "directly.",
+                category=DeprecationWarning,
+            )
+
         if config["CACHE_OPTIONS"]:
             cache_options.update(config["CACHE_OPTIONS"])
 
@@ -233,37 +248,37 @@ class Cache(object):
             app.extensions = {}
 
         app.extensions.setdefault("cache", {})
-        app.extensions["cache"][self] = cache_obj(
+        app.extensions["cache"][self] = cache_factory(
             app, config, cache_args, cache_options
         )
         self.app = app
 
     @property
-    def cache(self):
+    def cache(self) -> SimpleCache:
         app = current_app or self.app
         return app.extensions["cache"][self]
 
-    def get(self, *args, **kwargs):
+    def get(self, *args, **kwargs) -> Optional[Union[str, Markup]]:
         """Proxy function for internal cache object."""
         return self.cache.get(*args, **kwargs)
 
-    def set(self, *args, **kwargs):
+    def set(self, *args, **kwargs) -> bool:
         """Proxy function for internal cache object."""
         return self.cache.set(*args, **kwargs)
 
-    def add(self, *args, **kwargs):
+    def add(self, *args, **kwargs) -> bool:
         """Proxy function for internal cache object."""
         return self.cache.add(*args, **kwargs)
 
-    def delete(self, *args, **kwargs):
+    def delete(self, *args, **kwargs) -> bool:
         """Proxy function for internal cache object."""
         return self.cache.delete(*args, **kwargs)
 
-    def delete_many(self, *args, **kwargs):
+    def delete_many(self, *args, **kwargs) -> bool:
         """Proxy function for internal cache object."""
-        return self.cache.delete_many(*args, **kwargs)
+        return self.cache.delete_many(*args, **kwargs)  # type: ignore
 
-    def clear(self):
+    def clear(self) -> None:
         """Proxy function for internal cache object."""
         return self.cache.clear()
 
@@ -279,7 +294,7 @@ class Cache(object):
         """Proxy function for internal cache object."""
         return self.cache.get_dict(*args, **kwargs)
 
-    def unlink(self, *args, **kwargs):
+    def unlink(self, *args, **kwargs) -> bool:
         """Proxy function for internal cache object
         only support Redis
         """
@@ -290,17 +305,17 @@ class Cache(object):
 
     def cached(
         self,
-        timeout=None,
-        key_prefix="view/%s",
-        unless=None,
-        forced_update=None,
-        response_filter=None,
-        query_string=False,
-        hash_method=hashlib.md5,
-        cache_none=False,
-        make_cache_key=None,
-        source_check=None
-    ):
+        timeout: Optional[int] = None,
+        key_prefix: str = "view/%s",
+        unless: Optional[Callable] = None,
+        forced_update: Optional[Callable] = None,
+        response_filter: Optional[Callable] = None,
+        query_string: bool = False,
+        hash_method: Callable = hashlib.md5,
+        cache_none: bool = False,
+        make_cache_key: Optional[Callable] = None,
+        source_check: Optional[bool] = None,
+    ) -> Callable:
         """Decorator. Use this to cache a function. By default the cache key
         is `view/request.path`. You are able to use this decorator with any
         function by changing the `key_prefix`. If the token `%s` is located
@@ -417,8 +432,9 @@ class Cache(object):
                     if make_cache_key is not None and callable(make_cache_key):
                         cache_key = make_cache_key(*args, **kwargs)
                     else:
-                        cache_key = _make_cache_key(args, kwargs, use_request=True)
-
+                        cache_key = _make_cache_key(
+                            args, kwargs, use_request=True
+                        )
 
                     if (
                         callable(forced_update)
@@ -535,14 +551,17 @@ class Cache(object):
                         if use_request:
                             cache_key = key_prefix % request.path
                         else:
-                            cache_key = key_prefix % url_for(f.__name__, **kwargs)
+                            cache_key = key_prefix % url_for(
+                                f.__name__, **kwargs
+                            )
                     else:
                         cache_key = key_prefix
 
                 if source_check and callable(f):
                     func_source_code = inspect.getsource(f)
                     func_source_hash = hash_method(
-                        func_source_code.encode("utf-8"))
+                        func_source_code.encode("utf-8")
+                    )
                     func_source_hash = str(func_source_hash.hexdigest())
 
                     cache_key += func_source_hash
@@ -557,22 +576,23 @@ class Cache(object):
 
         return decorator
 
-    def _memvname(self, funcname):
+    def _memvname(self, funcname: str) -> str:
         return funcname + "_memver"
 
-    def _memoize_make_version_hash(self):
+    def _memoize_make_version_hash(self) -> str:
         return base64.b64encode(uuid.uuid4().bytes)[:6].decode("utf-8")
 
     def _memoize_version(
         self,
-        f,
-        args=None,
+        f: Callable,
+        args: Optional[Any] = None,
         kwargs=None,
-        reset=False,
-        delete=False,
-        timeout=None,
-        forced_update=False,
-    ):
+        reset: bool = False,
+        delete: bool = False,
+        timeout: Optional[int] = None,
+        forced_update: Optional[Union[bool, Callable]] = False,
+        args_to_ignore: Optional[Any] = None,
+    ) -> Union[Tuple[str, str], Tuple[str, None]]:
         """Updates the hash version associated with a memoized function or
         method.
         """
@@ -580,6 +600,10 @@ class Cache(object):
         version_key = self._memvname(fname)
         fetch_keys = [version_key]
 
+        args_to_ignore = args_to_ignore or []
+        if "self" in args_to_ignore:
+            instance_fname = None
+
         if instance_fname:
             instance_version_key = self._memvname(instance_fname)
             fetch_keys.append(instance_version_key)
@@ -629,18 +653,24 @@ class Cache(object):
 
     def _memoize_make_cache_key(
         self,
-        make_name=None,
-        timeout=None,
-        forced_update=False,
-        hash_method=hashlib.md5,
-        source_check=False
-    ):
+        make_name: None = None,
+        timeout: Optional[Callable] = None,
+        forced_update: bool = False,
+        hash_method: Callable = hashlib.md5,
+        source_check: Optional[bool] = False,
+        args_to_ignore: Optional[Any] = None,
+    ) -> Callable:
         """Function used to create the cache_key for memoized functions."""
 
         def make_cache_key(f, *args, **kwargs):
             _timeout = getattr(timeout, "cache_timeout", timeout)
             fname, version_data = self._memoize_version(
-                f, args=args, timeout=_timeout, forced_update=forced_update
+                f,
+                args=args,
+                kwargs=kwargs,
+                timeout=_timeout,
+                forced_update=forced_update,
+                args_to_ignore=args_to_ignore,
             )
 
             #: this should have to be after version_data, so that it
@@ -649,7 +679,7 @@ class Cache(object):
 
             if callable(f):
                 keyargs, keykwargs = self._memoize_kwargs_to_args(
-                    f, *args, **kwargs
+                    f, *args, **kwargs, args_to_ignore=args_to_ignore
                 )
             else:
                 keyargs, keykwargs = args, kwargs
@@ -673,13 +703,14 @@ class Cache(object):
 
         return make_cache_key
 
-    def _memoize_kwargs_to_args(self, f, *args, **kwargs):
+    def _memoize_kwargs_to_args(self, f: Callable, *args, **kwargs) -> Any:
         #: Inspect the arguments to the function
         #: This allows the memoization to be the same
         #: whether the function was called with
         #: 1, b=2 is equivilant to a=1, b=2, etc.
         new_args = []
         arg_num = 0
+        args_to_ignore = kwargs.pop("args_to_ignore", None) or []
 
         # If the function uses VAR_KEYWORD type of parameters,
         # we need to pass these further
@@ -689,7 +720,10 @@ class Cache(object):
 
         for i in range(args_len):
             arg_default = get_arg_default(f, i)
-            if i == 0 and arg_names[i] in ("self", "cls"):
+            if arg_names[i] in args_to_ignore:
+                arg = None
+                arg_num += 1
+            elif i == 0 and arg_names[i] in ("self", "cls"):
                 #: use the id func of the class instance
                 #: this supports instance methods for
                 #: the memoized functions, giving more
@@ -740,7 +774,9 @@ class Cache(object):
             ),
         )
 
-    def _bypass_cache(self, unless, f, *args, **kwargs):
+    def _bypass_cache(
+        self, unless: Optional[Callable], f: Callable, *args, **kwargs
+    ) -> bool:
         """Determines whether or not to bypass the cache by calling unless().
         Supports both unless() that takes in arguments and unless()
         that doesn't.
@@ -762,15 +798,16 @@ class Cache(object):
 
     def memoize(
         self,
-        timeout=None,
-        make_name=None,
-        unless=None,
-        forced_update=None,
-        response_filter=None,
-        hash_method=hashlib.md5,
-        cache_none=False,
-        source_check=None
-    ):
+        timeout: Optional[int] = None,
+        make_name: None = None,
+        unless: None = None,
+        forced_update: Optional[Callable] = None,
+        response_filter: None = None,
+        hash_method: Callable = hashlib.md5,
+        cache_none: bool = False,
+        source_check: Optional[bool] = None,
+        args_to_ignore: Optional[Any] = None,
+    ) -> Callable:
         """Use this to cache the result of a function, taking its arguments
         into account in the cache key.
 
@@ -847,9 +884,17 @@ class Cache(object):
                              formed with the function's source code hash in
                              addition to other parameters that may be included
                              in the formation of the key.
+        :param args_to_ignore: List of arguments that will be ignored while
+                               generating the cache key. Default to None.
+                               This means that those arguments may change
+                               without affecting the cache value that will be
+                               returned.
 
         .. versionadded:: 0.5
             params ``make_name``, ``unless``
+
+        .. versionadded:: 1.10
+            params ``args_to_ignore``
         """
 
         def memoize(f):
@@ -928,7 +973,8 @@ class Cache(object):
                 timeout=decorated_function,
                 forced_update=forced_update,
                 hash_method=hash_method,
-                source_check=source_check
+                source_check=source_check,
+                args_to_ignore=args_to_ignore,
             )
             decorated_function.delete_memoized = lambda: self.delete_memoized(f)
 
@@ -936,7 +982,7 @@ class Cache(object):
 
         return memoize
 
-    def delete_memoized(self, f, *args, **kwargs):
+    def delete_memoized(self, f, *args, **kwargs) -> None:
         """Deletes the specified functions caches, based by given parameters.
         If parameters are given, only the functions that were memoized
         with them will be erased. Otherwise all versions of the caches
@@ -1054,7 +1100,7 @@ class Cache(object):
             cache_key = f.make_cache_key(f.uncached, *args, **kwargs)
             self.cache.delete(cache_key)
 
-    def delete_memoized_verhash(self, f, *args):
+    def delete_memoized_verhash(self, f: Callable, *args) -> None:
         """Delete the version hash associated with the function.
 
         .. warning::
